//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const FLOCK_BLOG_NS = "http://www.flock.com/rdf#";
const FLOCK_BLOG_STATE_NEW = "NEW";
const FLOCK_BLOG_STATE_CHANGED = "CHANGED";
const FLOCK_BLOG_STATE_DRAFT = "DRAFT";
const FLOCK_BLOG_STATE_OK = "OK";
const FLOCK_BLOG_DEFAULT_TITLE = "Untitled";

const FLOCK_BLOG_RDF_FILE = "flock_blog.rdf";
const FLOCK_BLOG_RDF_SETTINGS = "urn:flock:blog:settings";
const FLOCK_BLOG_RDF_TMP_POST = "urn:flock:blog:tmp:posts";
const FLOCK_BLOG_RDF_ALLPOSTS_FOLDER = "urn:flock:blog:AllPosts";
const FLOCK_BLOG_RDF_PREF_ROOT = "urn:flock:blog:preferences";
const FLOCK_BLOG_RDF_NOTIFICATION_ROOT = FLOCK_BLOG_RDF_PREF_ROOT + ":notifications";
const FLOCK_BLOG_RDF_RECENT_ROOT = "urn:flock:blog:recent";

const IBLOG_SERVICE = Components.interfaces['flockIBlogService'];
const IBLOG_WEB_SERVICE = Components.interfaces['flockIBlogWebService'];
const BLOG_SERVICE = '@mozilla.org/rdf/datasource;1?name=flock-blog';
const BLOG_ATOM = '@flock.com/blog/service/atom;1';
const BLOG_MOVABLETYPE = '@flock.com/blog/service/movabletype;1';
const BLOG_BLOGGER = '@flock.com/blog/service/blogger;1';


/////////////////////////////////////////////////////////////////
// General Data management - Settings, Accounts, Posts
/////////////////////////////////////////////////////////////////

var gBlogSettings = null;

flock.blog.__defineGetter__("settings", function () {  	 	 
    if(!gBlogSettings) { 		 
        gBlogSettings = new BlogSettings(); 		 
        gBlogSettings.init(); 		 
    } 		 
    return gBlogSettings; 		 
});

function flock_blogGetPassword(aHost, aUsername) {
    var psm = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
    var enum = psm.enumerator;
    while(enum.hasMoreElements()) {
        var pw = enum.getNext();
        pw = pw.QueryInterface(Components.interfaces.nsIPassword);
        if(pw.host==aHost && aUsername==pw.user) {
            return pw.password;
        }
    }
    return null;
}

// TODO: Move all the RDF related stuff to the XPCOM component
// + remove the unused stuff
function BlogSettings() { }

BlogSettings.prototype =  {
    init: function () {
        var fname = FLOCK_BLOG_RDF_FILE;

        var blogFile = Components.classes["@mozilla.org/file/directory_service;1"].
            getService(Components.interfaces.nsIProperties).
            get("ProfD", Components.interfaces.nsIFile);

        blogFile.append(fname);

        if(!blogFile.exists()) {
          blogFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
        }

        var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
                                  .getService(Components.interfaces.nsIIOService);
                                  
        var fileHandler = ioSvc.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
        var furl = fileHandler.getURLSpecFromFile(blogFile);

        var ds = this.ds = new RDFDataSource(furl);

        var root=ds.getNode("urn:flock:blog");
        var settings = ds.getNode(FLOCK_BLOG_RDF_SETTINGS);
        settings.makeSeq();

        var notif = ds.getNode(FLOCK_BLOG_RDF_NOTIFICATION_ROOT);
        notif.makeSeq();

        var firstRun = flock_getIntPref("flock.blog.firstRun");
        if(!firstRun) {
            this.addNotification("http://www.technorati.com")
            flock_setIntPref("flock.blog.firstRun", 1);
            flock_setBoolPref("flock.blog.notify", true);
        }

        //this.getAllPostsContainer();
        //this.rebuildAllPosts();
        this.ds.save();
    },
    needRefresh: function () {
        var ds = this.ds;
        var settings = ds.getNode(FLOCK_BLOG_RDF_SETTINGS);
        var profiles = settings.getChildren();
        var needRefresh = false;

        while(profiles.hasMoreElements()) {
            var profile = profiles.getNext();
            var title = profile.getTarget(FLOCK_BLOG_NS+"title").getValue();
            var init = profile.getTarget(FLOCK_BLOG_NS+"init");
            if(!init || init.getValue()!="1")
            {
                needRefresh = true;
            }
        }
        return needRefresh;
    },
    getAccountTitle: function (aAccount) {
        var ds = this.ds;
        var account = ds.getNode(aAccount);
        return account.getTarget(FLOCK_BLOG_NS+"title").getValue();
    },

   
    getSettingsContainer: function (aBlogAccount) {
        var settings = this.ds.getNode(FLOCK_BLOG_RDF_SETTINGS);
        settings.makeSeq();
        return settings;
    },

    getAccount: function (aBlogID) {
        if(!aBlogID) throw new Error("Error in BlogSettings.getAccount()");
        var ds = this.ds;
        var rval = new BlogAccount();
        var account = ds.getNode(aBlogID);
        try {
        rval.title = account.getTarget(FLOCK_BLOG_NS+"title").getValue();
        rval.username = account.getTarget(FLOCK_BLOG_NS+"username").getValue();
        rval.pwhost = account.getTarget(FLOCK_BLOG_NS+"pwhost").getValue();
//        alert("pre" + rval.username + " " + rval.pwhost);
        rval.password = flock_blogGetPassword(rval.pwhost, rval.username);
//        alert("post" + rval.password);
        //psm.addUser(pwhost, aBlogAccount.username, aBlogAccount.password);
//        rval.password = account.getTarget(FLOCK_BLOG_NS+"password").getValue();
        rval.uid = aBlogID;
        rval.url = account.getTarget(FLOCK_BLOG_NS+"url").getValue();
        } catch(e) {}
        return rval;
    },

    getAllPostsContainer: function() {
        var allPosts = this.ds.getNode(FLOCK_BLOG_RDF_ALLPOSTS_FOLDER);
        allPosts.makeSeq();
        return allPosts;

    },

    rebuildAllPosts: function() {
        var ds = this.ds;
        this.clearAllPosts();
        var allPosts = this.getAllPostsContainer();

        var settings = ds.getNode(FLOCK_BLOG_RDF_SETTINGS);
        var profiles = settings.getChildren();
        while(profiles.hasMoreElements()) {
            var profile = profiles.getNext();
            var list = profile.getTarget(FLOCK_BLOG_NS+"posts");
            var enum = list.getChildren();
            while(enum.hasMoreElements()) {
                var post = enum.getNext();
                allPosts.addChild(post);
            }
        }
    },

    clearAllPosts: function() {
        try {
            var ds = this.ds;
            var dsource = ds.getRawDataSource();
            var node = ds.getNode(FLOCK_BLOG_RDF_ALLPOSTS_FOLDER).source;
            var props=ds.getRawDataSource().ArcLabelsOut(node);
            while(props.hasMoreElements()){
                var prop=props.getNext();
                var target=dsource.GetTarget(node,prop,true);
                dsource.Unassert(node,prop,target)
            }

        } catch(e) {}
    },

    addNotification: function(aURL) {
        var ds = this.ds;
        var root = ds.getNode(FLOCK_BLOG_RDF_NOTIFICATION_ROOT);
        root.makeSeq();
        var node = ds.getNode(FLOCK_BLOG_RDF_NOTIFICATION_ROOT + ":" + aURL);
        node.addTargetOnce(FLOCK_BLOG_NS + "url", aURL);
        root.addChild(node);
        ds.save();
    },

    getNotifications: function() {
        var ds = this.ds;
        var root = ds.getNode(FLOCK_BLOG_RDF_NOTIFICATION_ROOT);
        var rval = new Array();
        var enum = root.getChildren();
        while(enum.hasMoreElements()) {
            var item = enum.getNext();
            var notification  = item.getTarget(FLOCK_BLOG_NS + "url").getValue();
            rval.push(notification);
        }
        return rval;
    },

    removeNotification: function(aURL) {
        var ds = this.ds;
        var root = ds.getNode(FLOCK_BLOG_RDF_NOTIFICATION_ROOT);
        var node = ds.getNode(FLOCK_BLOG_RDF_NOTIFICATION_ROOT + ":" + aURL);
        ds.deleteRecursive(node);
        root.removeChild(node);
        ds.save();
    },

    getAccountCount: function () {
        var ds = this.ds;
        var settings = ds.getNode(FLOCK_BLOG_RDF_SETTINGS);
        var profiles = settings.getChildren();
        var count = 0;
        while(profiles.hasMoreElements()) {
            var post = profiles.getNext();
            ++count;
        }
        return count;
    },
    ds: null,
}

function BlogAccount(aTitle) {
    this.title = aTitle;
    this.uid = FLOCK_BLOG_RDF_SETTINGS + ":" + aTitle;
    this.ds = flock.blog.settings.ds;
    this.settings = flock.blog.settings;
    this.url = " ";
}

BlogAccount.prototype =  {

    getAPINode: function (aBlogPost) {
        //dump(this.uid + ":" + "api");
        return this.ds.getNode(this.uid + ":" + "api");
    },
    getCategoriesRef: function (a) {
        return this.uid + ":" + "categories";
    },
    replace: function(aIn, aRegExp, aReplace, aOpts) {
        if (!aOpts) aOpts = "g";

        var re = new RegExp(aRegExp, aOpts);
        return aIn.replace(re, aReplace);
    },
    ampString: "aMp34SAnd",//ugh
    savePost: function (aBlogPost, aDontSave) {
        var postid = aBlogPost.getPostID();
        var bloguri = aBlogPost.getBlogURI();

        var ds = this.ds;
        var post = ds.getNode(aBlogPost.uri);

        if(!title) title = FLOCK_BLOG_DEFAULT_TITLE;
        post.addTargetOnce(FLOCK_BLOG_NS+"title", aBlogPost.title + "");
        post.addTargetOnce(FLOCK_BLOG_NS+"postid", aBlogPost.uid + "");
        post.addTargetOnce(FLOCK_BLOG_NS+"date", aBlogPost.dateCreated + "");
        var dateUI = padDateTime(aBlogPost.dateCreated.getTime());
        post.addTargetOnce(FLOCK_BLOG_NS+"dateUI", dateUI + "");
        post.addTargetOnce(FLOCK_BLOG_NS+"state", aBlogPost.state + "");
        var content = aBlogPost.content + "";
		content = this.replace(content, "&", this.ampString, "g");
        post.addTargetOnce(FLOCK_BLOG_NS+"content", content);
        post.addTargetOnce(FLOCK_BLOG_NS+"tags", aBlogPost.tags + "");
        post.addTargetOnce(FLOCK_BLOG_NS+"editURI", aBlogPost.editURI + "");
        post.addTargetOnce(FLOCK_BLOG_NS+"atomid", aBlogPost.atomid + "");
        if(aBlogPost.permaLink) post.addTargetOnce(FLOCK_BLOG_NS+"permaLink", aBlogPost.permaLink + "");

        if(aBlogPost.uri==FLOCK_BLOG_RDF_TMP_POST) return;

        var posts = this.getPostsContainer(bloguri);
        posts.addChild(post);

        this.settings.getAllPostsContainer().addChild(post);

        if(!aDontSave) ds.save();
    },

    flushCategories: function () {
        var ds = this.ds;
        var categoriesNode = ds.getNode(this.getCategoriesRef());
        flock_emptyContainer(ds, categoriesNode);
    },

    addCategory: function (aCategoryId, aCategoryName) {
        var ds = this.ds;
        var categoriesNode = ds.getNode(this.getCategoriesRef());
        categoriesNode.makeSeq();
        var categoryUID = this.getCategoriesRef() + ":" + aCategoryId;
        var categoryNode = ds.getNode(categoryUID);
        categoryNode.addTargetOnce(FLOCK_BLOG_NS+"categoryName", aCategoryName + "");
        categoryNode.addTargetOnce(FLOCK_BLOG_NS+"categoryId", aCategoryId + "");
        categoriesNode.addChild(categoryNode);
        ds.save();
        //return this.ds.getNode(this.uid);
    },

    getAccountNode: function () {
        return this.ds.getNode(this.uid);
    },

    getPost: function (aPostID) {
        var blogPost = new BlogPost(this.uid, aPostID);
        return this.getPostByURI(blogPost.uri);
    },

    getPostByURI: function (aPostURI) {
        var blogPost = new BlogPost();
        blogPost.uri = aPostURI;
        var uri = aPostURI;
        var ds = this.ds;
        var post = ds.getNode(blogPost.uri);
        try
        {
            blogPost.title = post.getTarget(FLOCK_BLOG_NS+"title").getValue();
            blogPost.state = post.getTarget(FLOCK_BLOG_NS+"state").getValue();
            var content = post.getTarget(FLOCK_BLOG_NS+"content").getValue() + "";
		    blogPost.content = this.replace(content, this.ampString, "&", "g");
            blogPost.dateCreated = new Date(post.getTarget(FLOCK_BLOG_NS+"date").getValue());
            blogPost.tags = post.getTarget(FLOCK_BLOG_NS+"tags").getValue();
            blogPost.editURI = post.getTarget(FLOCK_BLOG_NS+"editURI").getValue();
            blogPost.atomid = post.getTarget(FLOCK_BLOG_NS+"atomid").getValue();
//            blogPost.editURI = post.getTarget(FLOCK_BLOG_NS+"editURI").getValue();
//            blogPost.uid = aPostID;
            var permaLink = post.getTarget(FLOCK_BLOG_NS+"permaLink");
            if(permaLink) blogPost.permaLink = permaLink.getValue();
            blogPost.uri = uri;
        } catch(e) {
            return null;
        }

        return blogPost;
    },

    deletePost: function (aBlogPost) {
        var ds = this.ds;
        var post = ds.getNode(aBlogPost.uri);
        this.settings.getAllPostsContainer().removeChildForReal(post);

        var container = this.getPostsContainer(aBlogPost.getBlogURI());
        container.removeChildForReal(post);

        ds.deleteRecursive(post);
        this.settings.rebuildAllPosts();
        ds.save();
    },

    getBlogAPINode: function (aBlogAPI) {
        return this.ds.getNode(this.uid + ":" + "api");
    },

    setBlogAPI: function (aBlogAPI) {
        var ds = this.ds;
        var api = this.getBlogAPINode();
        //dump(FLOCK_BLOG_NS+"apiname "+ aBlogAPI.name + "\n");
        //dump(FLOCK_BLOG_NS+"apiLink "+ aBlogAPI.apiLink + "\n");
        //dump(FLOCK_BLOG_NS+"editLink "+ aBlogAPI.editLink + "\n");
        //dump(FLOCK_BLOG_NS+"blogid "+ aBlogAPI.blogID + "\n");
        api.addTargetOnce(FLOCK_BLOG_NS+"apiname", aBlogAPI.name + "");
        api.addTargetOnce(FLOCK_BLOG_NS+"apiLink", aBlogAPI.apiLink + "");
        api.addTargetOnce(FLOCK_BLOG_NS+"editLink", aBlogAPI.editLink + "");
        api.addTargetOnce(FLOCK_BLOG_NS+"blogid", aBlogAPI.blogID + "");
        ds.save();
    },

    getBlogAPI: function () {
        try {
            var ds = this.ds;
            var profile = ds.getNode(this.uid);
            var u = profile.getTarget(FLOCK_BLOG_NS+"username").getValue();
            var t = profile.getTarget(FLOCK_BLOG_NS+"title").getValue();
            var pwhost = profile.getTarget(FLOCK_BLOG_NS+"pwhost").getValue();
            var p = flock_blogGetPassword(pwhost, u);
            //alert(p);
            //var p = profile.getTarget(FLOCK_BLOG_NS+"password").getValue();

            var api = profile.getTarget(FLOCK_BLOG_NS+"api");
            var apiname = api.getTarget(FLOCK_BLOG_NS+"apiname").getValue();
            var apiLink = api.getTarget(FLOCK_BLOG_NS+"apiLink").getValue();
            var blogid = api.getTarget(FLOCK_BLOG_NS+"blogid").getValue();
            //dump("getBlogAPI: found API "+apiname+"\n");
            //dump("getBlogAPI: found apiLink "+apiLink+"\n");
            var B = null;

            B = flock.blog.createAPIFromName(apiname);
            B.init(apiname, "", apiLink, blogid, "", "");
            B.setAuth(u,p);

            return B;
        }
        catch(e) {
            dump("Caugth exception: "+e+" "+e.fileName+" "+e.lineNumber+"\n");
            return null;
        }
    },

/*    getPostsContainerURL: function (aURI) {
        if(aURI) return aURI;
        else return this.uid + ":" + "posts";
    },

    getPostsContainer: function (aURI) {
        var ds = this.ds;
        var posts = ds.getNode(this.getPostsContainerURL(aURI));
//        posts.makeSeq();
        return posts;

    }, */

    setInitialized: function (aVal) {
        var account = this.getAccountNode();
        account.addTargetOnce(FLOCK_BLOG_NS+"init", aVal + "");
    },

    isInitialized: function () {
        try {
            var val = this.getAccountNode().getTarget(FLOCK_BLOG_NS+"init").getValue();
            if(val=="1") return true;
            if(val=="true") return true;
            return false;
        } 
        catch(e) {
            return false;
        }
    },

    uid: "",
    username: "",
    title: "",
    password: "",
    //postURI: "",
    blogAPI: null,
    //blogURL: "",
    settings: null,
}

function BlogPost(aAccountID, aPostID) {
//    if(!aAccountID || !aPostID) throw new Error("error");
//    this.uri = this.getPostURI(aPostID);
    this.init(aAccountID, aPostID);
    //this.ds = flock_blogGetSettings().ds;
    this.ds = flock.blog.settings.ds;
}

BlogPost.prototype =  {
    getBlogURI: function () {
        this.uri.match(/(.+):(.*)/);
        return RegExp.$1;
    },
    getPostID: function () {
        this.uri.match(/(.+):(.*)/);
        return RegExp.$2;
    },
    getPostNode: function () {
        return this.ds.getNode(this.uri);
    },
    getPostURI: function (aPostID) {
        return this.uri + ":posts:" + aPostID;
    },
    getState: function () {
        try {
            var node = this.getPostNode();
            return node.getTarget(FLOCK_BLOG_NS+"state").getValue();
        } catch(e) {
            return FLOCK_BLOG_STATE_NEW;
        }
    },
    setState: function (aState) {
        try {
            var node = this.getPostNode();
            node.addTargetOnce(FLOCK_BLOG_NS+"state", aState);
            this.state = state;
        } catch(e) {
        }
    },
    init: function (aAccountID, aPostID) {
        if(!aAccountID || !aPostID) {
            this.uri = "";
            this.uid = "";
        }
        else {
            this.uri = aAccountID + ":posts:" + aPostID;
            this.uid = aPostID;
        }
    },
    editURI: "",
    title: "",
    content: "",
    dateCreated: "",
    state: FLOCK_BLOG_STATE_NEW,
    tags: "",
    uid: "",
    uri: "",
    accountID: "",
    atomid: "",
}

/////////////////////////////////////////////////////////////////
// API Objects
/////////////////////////////////////////////////////////////////

function BlogAPIListener() {
}

BlogAPIListener.prototype = {
    onResult: function(result) {},
    onFault: function (fault) {},
    onError: function (error) {}
};

var BlogAPI= function () {}

BlogAPI.prototype = {
    init: function (aName, aAppKey, aAPILink, aBlogID, aPreferred, aUsername, aPassword) {
        this.name = aName;
        this.appKey = aAppKey;
        this.editLink = "";
        this.apiLink = aAPILink;
        //this.appKey = "";
        this.blogID = aBlogID;
        this.username = aUsername;
        this.password = aPassword;
    },
    setAuth: function (aUsername, aPassword)
    {
        this.username = aUsername;
        this.password = aPassword;
    },
    newPost: function (aListener, aTitle, aDescription, aDateCreated, aPublish) {},
    editPost: function (aListener, aPostid, aTitle, aDescription, aDateCreated, aPublish) {},
    deletePost: function (aListener, aPostid) {},
    getPost: function (aListener, aPostid) {},
    getUsersBlogs: function (aListener) {},
    getRecentPosts: function (aListener, aNumber) {},
    setPostCategories: function (aListener, aPostid, aCategories) {},
    getCategoryList: function (aListener) {},
    getNotifications: function () {
        return flock.blog.settings.getNotifications();
    },
}

var BloggerAPI = function () {
    this.newPost = function (aListener, aTitle, aDescription, aDateCreated, aPublish) {
        return blogger_newPost(aListener, this.apiLink, this.appKey, this.blogID, this.username, this.password, aDescription, aPublish);
    };
    this.editPost = function (aListener, aPostid, aTitle, aDescription, aDateCreated, aPublish) {
        return blogger_editPost(aListener, this.apiLink, this.appKey, aPostid, this.username, this.password, aDescription, aPublish);
    };
    this.deletePost = function (aListener, aPostid) {
        return blogger_deletePost(aListener, this.apiLink, this.appKey, aPostid, this.username, this.password, true);
    };
    this.getPost = function (aListener, aPostid) {
        throw new Error("blogger getPost() not implemented");
    };
    this.getUsersBlogs = function (aListener) {
        return blogger_getUsersBlogs(aListener, this.apiLink, this.appKey, this.username, this.password);
    };
    this.getRecentPosts = function (aListener, aNumber) {
        return blogger_getRecentPosts(aListener, this.apiLink, this.appKey, this.blogID, this.username, this.password, aNumber);
    };
    this.getCategoryList = function (aListener) {
        aListener.onResult(null);
    };
    this.setPostCategories = function (aListener, aPostid, aCategories) {
        aListener.onResult(null);
    };
}
BloggerAPI.prototype = new BlogAPI();

var MetaweblogAPI = function () {
    this.newPost = function (aListener, aTitle, aDescription, aDateCreated, aPublish) {
        var content_t =
        {
            title: aTitle,
            description: aDescription,
            mt_convert_breaks: "false",
            mt_tb_ping_urls: this.getNotifications(),
        };
        return metaWeblog_newPost(aListener, this.apiLink, this.blogID, this.username, this.password, content_t, aPublish);
    };
    this.editPost = function (aListener, aPostid, aTitle, aDescription, aDateCreated, aPublish) {
        var notificationsArray = this.getNotifications();
        var content_t =
        {
            title: aTitle,
            description: aDescription,
            dateCreated: new Date(aDateCreated),
            mt_convert_breaks: "false",
            mt_tb_ping_urls: this.getNotifications(),
        };
        return metaWeblog_editPost(aListener, this.apiLink, aPostid, this.username, this.password, content_t, aPublish);
    };
    this.deletePost = function (aListener, aPostid) {
        return blogger_deletePost(aListener, this.apiLink, this.appKey, aPostid, this.username, this.password, true);
    };
    this.getPost = function (aListener, aPostid) {
        return metaWeblog_getPost(aListener, this.apiLink, aPostid, this.username, this.password);
    };
    this.getUsersBlogs = function (aListener) {
        return blogger_getUsersBlogs(aListener, this.apiLink, this.appKey, this.username, this.password);
    };
    this.getRecentPosts = function (aListener, aNumber) {
        return metaWeblog_getRecentPosts(aListener, this.apiLink, this.blogID, this.username, this.password, aNumber);
        //return blogger_getRecentPosts(aListener, this.apiLink, this.appKey, this.blogID, this.username, this.password, aNumber);
    };
    this.getCategoryList = function (aListener) {
        aListener.onResult(null);
    };
    this.setPostCategories = function (aListener, aPostid, aCategories) {
        aListener.onResult(null);
    };
}
MetaweblogAPI.prototype = new BlogAPI();


var MovableTypeAPI = function () {
    this.supportsCategories = true;
    this.newPost = function (aListener, aTitle, aDescription, aDateCreated, aPublish) {
        var content_t =
        {
            title: aTitle,
            description: aDescription,
            mt_convert_breaks: "",
            mt_tb_ping_urls: this.getNotifications(),
        };
        return metaWeblog_newPost(aListener, this.apiLink, this.blogID, this.username, this.password, content_t, aPublish);
    };
    this.editPost = function (aListener, aPostid, aTitle, aDescription, aDateCreated, aPublish) {
        var notificationsArray = this.getNotifications();
        var content_t =
        {
            title: aTitle,
            description: aDescription,
            dateCreated: new Date(aDateCreated),
            mt_convert_breaks: "",
            mt_tb_ping_urls: this.getNotifications(),
        };
        return metaWeblog_editPost(aListener, this.apiLink, aPostid, this.username, this.password, content_t, aPublish);
    };
    this.getPost = function (aListener, aPostid) {
        return metaWeblog_getPost(aListener, this.apiLink, aPostid, this.username, this.password);
    };
    this.deletePost = function (aListener, aPostid) {
        return blogger_deletePost(aListener, this.apiLink, this.appKey, aPostid, this.username, this.password, true);
    };
    this.getUsersBlogs = function (aListener) {
        return blogger_getUsersBlogs(aListener, this.apiLink, this.appKey, this.username, this.password);
    };
    this.getRecentPosts = function (aListener, aNumber) {
        return metaWeblog_getRecentPosts(aListener, this.apiLink, this.blogID, this.username, this.password, aNumber);
    };
    this.getCategoryList = function (aListener) {
        return mt_getCategoryList(aListener, this.apiLink, this.blogID, this.username, this.password);
    };
    this.setPostCategories = function (aListener, aPostid, aCategories) {
        return mt_setPostCategories(aListener, this.apiLink, aPostid, this.username, this.password, aCategories);
    };
}
MovableTypeAPI.prototype = new BlogAPI();

var AtomAPI = function () {
    this.newPost = function (aListener, aTitle, aDescription, aDateCreated, aPublish) {
        var J = new JSAtomAPI();
        J.init(this.apiLink);
        J.newPost(aListener, aTitle, aDescription, aDateCreated, aPublish);
    };
    this.editPost = function (aListener, aPostid, aTitle, aDescription, aDateCreated, aPublish, aEditURI, aAtomID) {
        var J = new JSAtomAPI();
        J.init(this.apiLink);
        J.editPost(aListener, aPostid, aTitle, aDescription, aDateCreated, aPublish, aEditURI, aAtomID);
    };
    this.deletePost = function (aListener, aPostid, aEditURI) {
        var J = new JSAtomAPI();
        J.init(this.apiLink);
        J.deletePost(aListener, aPostid, aEditURI);
    };
    this.getPost = function (aListener, aPostid) {
        var J = new JSAtomAPI();
        J.init(this.apiLink);
        J.getPost(aListener, aPostid);
        //throw new Error("atomapi getPost not implemented");
        //return metaweblog_getPost(aListener, this.apiLink, this.appKey, aPostid, this.username, this.password, true);
    };
    this.getUsersBlogs = function (aListener) {
        var J = new JSAtomAPI();
        J.init(this.apiLink);
        J.getUsersBlogs(aListener);
    };
    this.getRecentPosts = function (aListener, aNumber) {
        var J = new JSAtomAPI();
        J.init(this.apiLink);
        J.getRecentPosts(aListener);
    };
    this.getCategoryList = function (aListener) {
        aListener.onResult(null);
    };
    this.setPostCategories = function (aListener, aPostid, aCategories) {
        aListener.onResult(null);
    };
}
AtomAPI.prototype = new BlogAPI();


function flock_emptyContainer(aDS, aContainer) {
    var ds = aDS;
    var dsource = ds.getRawDataSource();
    var node = aContainer;
    var props=ds.getRawDataSource().ArcLabelsOut(node);
    while(props.hasMoreElements()){
        var prop=props.getNext();
        var target=dsource.GetTarget(node,prop,true);
        dsource.Unassert(node,prop,target)
    }
}

