/*  *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code Shadows.com API client implementation.
 *
 * The Initial Developer of the Original Code is
 * Pluck Corporation. This code is Copyright (C) 2005, 2006
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Carl Brown (carlb@shadows.com)
 *   Duane Fields (dfields@pluck.com)
 *
 * ***** END LICENSE BLOCK ***** */

//Code from Shadows.com's parent library

const PREF_USERNAME = 'shadows.com.username';
const PREF_PASSWORD = 'shadows.com.password';
const PREF_BOOKMARK_MAXVERSION = 'shadows.com.bookmarks.maxversion';
const PREF_COLLECTION_MAXVERSION = 'shadows.com.collections.maxversion';


shadows.PrefsManager = {

	getBranch: function() {
        var prefsService = Components.classes["@mozilla.org/preferences-service;1"].
    				getService(Components.interfaces.nsIPrefService);
        // todo this must be runtime configurable for reuse
        return prefsService.getBranch("shadows.com.");
	},
	
	prefExists: function(pref) {
        return this.getBranch().prefHasUserValue(pref);
	},
	
    getPref: function(key, defValue) {
        // todo this must be runtime configurable for reuse
        var prefs = this.getBranch();
        //debug("Prefs called with: " + key + " = default " + defValue + " type of: " + typeof defValue + "\n");
        if (prefs.prefHasUserValue(key)) {
            var value;
            switch(prefs.getPrefType(key)) {
                // treat ints as strings so we can store LONGs
                case prefs.PREF_INT:
                case prefs.PREF_STRING:
                    value = prefs.getCharPref(key);
                    if (value == null || value.length == 0) value = defValue;
                    break;
                case prefs.PREF_BOOL:
                    value = prefs.getBoolPref(key);
                    break;
                default:
                    PluckDebug.fail("Unexpected pref type\n");
            }
            //debug("Prefs: " + key + " = " + value);
            return value;
        }
        //debug("Returning default value: " + defValue);
        if (typeof defValue == "number") {
            return defValue;
        }
        return defValue ? defValue : null;
    },

    isEnabled: function(key, defValue) {
        return this.getPref(key, defValue);
    },

    togglePref: function(key, defValue)  {
        //debug("key is " + key);
        //debug("enabled was = " + this.isEnabled(key, defValue));
        this.setPref(key, this.isEnabled(key, defValue) ? false : true);
        //debug("enabled now = " + this.isEnabled(key, defValue));
        return this.isEnabled(key, defValue);
    },


    setPref: function(key, value) {
        // todo this must be runtime configurable for reuse
        var prefs = this.getBranch();
        //debug("Setting " + key + "=" + value);
        PluckDebug.assert(value != null, "should not be null");
        var type;
        if (prefs.prefHasUserValue(key))
            type = prefs.getPrefType(key);
        else {
            switch (typeof value) {
                case "string": type = prefs.PREF_STRING; break;
                case "number": type = prefs.PREF_INT; break;
                case "boolean": type = prefs.PREF_BOOL; break;
                case "object": type = prefs.PREF_STRING; value = value.toString(); break;
                default: PluckDebug.fail("invalid typeof " + typeof value); break;
            }
        }

        switch(type) {
            case prefs.PREF_STRING:
                prefs.setCharPref(key, value);
                break;
            case prefs.PREF_BOOL:
                prefs.setBoolPref(key, value);
                break;
            case prefs.PREF_INT:
				//prefs.setIntPref(key, value);
				//NOTE: we still treat as strings, PREF_INT cannot handle longs
				prefs.setCharPref(key, value);
                break;
            default:
                PluckDebug.fail("Unexpected pref type " + prefs.getPrefType(key));
        }
    },

    clearPref: function(key) {
        // todo this must be runtime configurable for reuse
        var prefs = this.getBranch();
        try {
            prefs.clearUserPref(key);
        } catch (e) { debug("clearPref: " + e); }
    },
        
    isPluckDebugEnabled: function() { return this.isEnabled(PREF_DEBUG, false);},
    getUsername: function(def) { return this.getPref(PREF_USERNAME, def); },

    getPassword: function(def) {
        //var decoder = Components.classes["@mozilla.org/security/sdr;1"].
    	//			createInstance(Components.interfaces.nsISecretDecoderRing);
        var password = this.getPref(PREF_PASSWORD, def);
        if (password == null || password.length ==0)
           return def;
        try {
            //return decoder.decryptString(password);
            return shadows.CryptUtils.decrypt(password, '{4eac53f6-82b1-4051-8fca-fe064d127b83}');
        } catch (e) {
            return def;
        }
    },
    setUsername: function(username) { return this.setPref(PREF_USERNAME, username); },
    setPassword: function(password) {
        //var encoder = Components.classes["@mozilla.org/security/sdr;1"].
    	//			createInstance(Components.interfaces.nsISecretDecoderRing);
        //var crypted = encoder.encryptString(password);
        var crypted = shadows.CryptUtils.encrypt(password, '{4eac53f6-82b1-4051-8fca-fe064d127b83}');
        this.setPref(PREF_PASSWORD, crypted);
    },
    resetUsernamePassword: function() {
        this.clearPref(PREF_USERNAME);
        this.clearPref(PREF_PASSWORD);
    },
    getBookmarkVersion: function(def) {
        return parseInt (this.getPref(PREF_BOOKMARK_MAXVERSION, def));
    },
    setBookmarkVersion: function(version) { 
        //if parameter is bigger, save it
        if (version > this.getPref(PREF_BOOKMARK_MAXVERSION, 0)) {
            return this.setPref(PREF_BOOKMARK_MAXVERSION, version); 
        } else { return false; }
    },    
    resetBookmarkVersion: function() {
        this.clearPref (PREF_BOOKMARK_MAXVERSION);
    },
    getCollectionVersion: function(def) { 
        return parseInt (this.getPref(PREF_COLLECTION_MAXVERSION, def)); 
    },
    setCollectionVersion: function(version) { 
        //if parameter is bigger, save it
        if (version > this.getPref(PREF_COLLECTION_MAXVERSION, 0)) {
            return this.setPref(PREF_COLLECTION_MAXVERSION, version); 
        } else { return false; }
    },
    resetCollectionVersion: function() {
        this.clearPref (PREF_COLLECTION_MAXVERSION);
    }
};

shadows.CryptUtils = {
    encrypt: function(str, pwd) {
      if(pwd == null || pwd.length <= 0) {
        PluckDebug.fail("empty password key not allowed");
        return null;
      }
      var prand = "";
      for(var i=0; i<pwd.length; i++) {
        prand += pwd.charCodeAt(i).toString();
      }
      var sPos = Math.floor(prand.length / 5);
      var mult = parseInt(prand.charAt(sPos) + prand.charAt(sPos*2) + prand.charAt(sPos*3) + prand.charAt(sPos*4) + prand.charAt(sPos*5));
      var incr = Math.ceil(pwd.length / 2);
      var modu = Math.pow(2, 31) - 1;
      if(mult < 2) {
        PluckDebug.fail("invalid password key, unable to hash");
        return null;
      }
      var salt = Math.round(Math.random() * 1000000000) % 100000000;
      prand += salt;
      while(prand.length > 10) {
        prand = (parseInt(prand.substring(0, 10)) + parseInt(prand.substring(10, prand.length))).toString();
      }
      prand = (mult * prand + incr) % modu;
      var enc_chr = "";
      var enc_str = "";
      for(var i=0; i<str.length; i++) {
        enc_chr = parseInt(str.charCodeAt(i) ^ Math.floor((prand / modu) * 255));
        if(enc_chr < 16) {
          enc_str += "0" + enc_chr.toString(16);
        } else enc_str += enc_chr.toString(16);
        prand = (mult * prand + incr) % modu;
      }
      salt = salt.toString(16);
      while(salt.length < 8)salt = "0" + salt;
      enc_str += salt;
      return enc_str;
    },

    decrypt: function(str, pwd) {
      if(str == null || str.length < 8) {
        // A salt value could not be extracted from the encrypted message because it's length is too short. The message cannot be decrypted
        return null;
      }
      if(pwd == null || pwd.length <= 0) {
        PluckDebug.fail("empty password key not allowed");
        return;
      }
      var prand = "";
      for(var i=0; i<pwd.length; i++) {
        prand += pwd.charCodeAt(i).toString();
      }
      var sPos = Math.floor(prand.length / 5);
      var mult = parseInt(prand.charAt(sPos) + prand.charAt(sPos*2) + prand.charAt(sPos*3) + prand.charAt(sPos*4) + prand.charAt(sPos*5));
      var incr = Math.round(pwd.length / 2);
      var modu = Math.pow(2, 31) - 1;
      var salt = parseInt(str.substring(str.length - 8, str.length), 16);
      str = str.substring(0, str.length - 8);
      prand += salt;
      while(prand.length > 10) {
        prand = (parseInt(prand.substring(0, 10)) + parseInt(prand.substring(10, prand.length))).toString();
      }
      prand = (mult * prand + incr) % modu;
      var enc_chr = "";
      var enc_str = "";
      for(var i=0; i<str.length; i+=2) {
        enc_chr = parseInt(parseInt(str.substring(i, i+2), 16) ^ Math.floor((prand / modu) * 255));
        enc_str += String.fromCharCode(enc_chr);
        prand = (mult * prand + incr) % modu;
      }
      return enc_str;
    }
};

shadows.XmlUtils = {

    // xpath query that returns the results as an array of nodes
    evaluateXPath: function(aNode, aExpr) {
        var xpe = new XPathEvaluator();
        var nsResolver = xpe.createNSResolver(aNode.ownerDocument == null ?
            aNode.documentElement : aNode.ownerDocument.documentElement);
        var result = xpe.evaluate(aExpr, aNode, nsResolver, 0, null);
        var found = [];
        var res = result.iterateNext();
        while (res != null) {
            found.push(res);
            res = result.iterateNext();
        }
        return found;
    }

};



shadows.RdfUtils = {

    // convenience method to handle null noodes
     getNodeValue: function(node, defaultValue) {
        if (node == null) return defaultValue;;
        return node.getValue();
    }

};



shadows.Utils = {

     sleep: function(millis) {
        var then = new Date().getTime();
        var now = then;
        while ((now-then) < millis)
        {
            now = new Date().getTime();
        }
    },
	win32TicksFromDate: function(date) {
		var adjust = 0;
		var epoch = new Date();
		epoch.setTime(0);
		if (epoch.getFullYear() == 1969) {
			adjust = 3600000;
		}
		var ms = date.getTime()+adjust;
		var ticks = ms * 10000;
		ticks = ticks + 621355752000000000;
		return ticks;
	},
	dateFromWin32Ticks: function(ticks) {
		var adjust = 0;
		var epoch = new Date();
		epoch.setTime(0);
		if (epoch.getFullYear() == 1969) {
			adjust = -3600000;
		}
		ticks = ticks - 621355752000000000;
		var ms = ticks / 10000;
		ms = ms - adjust;
		var date = new Date();
		date.setTime(ms);
		return date;
	},
	// prompt user with title and question, then return yes or no answer
	askYesNo: function (title, question) {
		try {
			var promptService = Components.classes[ "@mozilla.org/embedcomp/prompt-service;1" ].
								getService(Components.interfaces.nsIPromptService);
			var promptFlags = promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0;
			promptFlags += promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1;

			var retVal = promptService.confirmEx( window, title, question,
							promptFlags, "button1", "button2", null, null, {value:0});
			if (0 == retVal) {
				return true;
			}
		}
		catch(e) {
			debug("askYesNo: "+e);
			throw e;
		}
		return false;
	},

	decimalToHex: function(decnum)
	{
		var hexnum="";
		var result=decnum;
		while (result>0)
		{
			result=Math.floor(decnum/16);
			var remainder = decnum % 16;
			decnum=result;
			
			if (remainder>=10)
			{
				if (remainder==10)
					remainder='A';
				if (remainder==11)
					remainder='B';
				if (remainder==12)
					remainder='C';
				if (remainder==13)
					remainder='D';
				if (remainder==14)
					remainder='E';
				if (remainder==15)
					remainder='F';
			}
			// just append the next remainder to the beginning of the string
			hexnum=""+remainder+hexnum;
		};
		return hexnum;
	},

	generateGuid: function() {
		var guid = "49fbcccd-597a-";

		suffix = Math.floor(Math.random() * 65535);
		suffix = "" + this.decimalToHex(suffix);
		if (suffix.length > 12) {
			suffix = suffix.substring(0, 4);
		}
		else while (suffix.length < 4) {
			suffix = "0" + suffix;
		}

		guid = guid + suffix + "-";

		suffix = Math.floor(Math.random() * 65535);
		suffix = "" + this.decimalToHex(suffix);
		if (suffix.length > 12) {
			suffix = suffix.substring(0, 4);
		}
		else while (suffix.length < 4) {
			suffix = "0" + suffix;
		}

		guid = guid + suffix + "-";

		var suffix = new Date().getTime();
		suffix = "" + this.decimalToHex(suffix);
		if (suffix.length > 12) {
			suffix = suffix.substring(0, 12);
		}
		else while (suffix.length < 12) {
			suffix = "0" + suffix;
		}

		guid = guid + suffix;
		
		debug("GenerateGuid: "+guid);
		
		return guid;
	},


	// are we logging server requests and responses to the jconsole?
	amLoggingServerCalls: function() {
        return PrefsManager.isEnabled(PREF_LOGSERVERCALLS, false);
	}
};

