// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const flockIFavoritesWebService = Components.interfaces.flockIFavoritesWebService;
const flockIFavoritesService = Components.interfaces.flockIFavoritesService;

const HTML_NS = 'http://www.w3.org/1999/xhtml'

var services = {};

// if cancel is clicked this is set. this allows callbacks to terminate correctly
var cancelled = false;

var chosenService = null;
var currentService = null;
var currentUser = '';

var usernameChanged = false;
var oldUsername = "";
var oldPassword = "";

var origService = null;
var origUsername = null;
var origPassword = null;

var firstrun = false;

// initialise the favorites sharing configuration ui
function init () {

    try {
      if (window.arguments[0] == 'firstrun') firstrun = true;
    } catch (e) {}

    // initialise the service selector
    var service, menuitem;
    var enabled = flock.favorites.service.webService?true:false;
    var svciter = flock.favorites.service.getWebServices ();
    var radiogroup = $('service');
    while (svciter.hasMoreElements ()) {
        service = svciter.getNext ().QueryInterface (flockIFavoritesWebService);
        services[service.id] = service;
        var hbox = document.createElement ('hbox');
        hbox.setAttribute('align', 'center');
        hbox.setAttribute('style', 'padding-bottom: 5px;');
        radiogroup.appendChild (hbox);
        var radio = document.createElement ('radio');
        radio.setAttribute ('id', service.id);
        radio.setAttribute ('oncommand', 'choose_service()');
        radio.radioGroup = radiogroup;
        hbox.appendChild (radio);
        var label = document.createElement ('label');
        label.setAttribute ('flex', '1');
        label.setAttribute ('control', service.id);
        label.setAttribute ('onclick', 'toggleServiceSelection(this);');
        hbox.appendChild (label);
        var p = document.createElementNS (HTML_NS, 'html:div');
        p.setAttribute ('style', 'width: 300px;');
        var b = document.createElementNS (HTML_NS, 'html:b');
        b.appendChild (document.createTextNode (service.name));
        p.appendChild (b);
        if (!firstrun) {
          p.appendChild (document.createTextNode (' '));
          p.appendChild (document.createTextNode (service.description));
          if (service.signupPage) {
            p.appendChild (document.createTextNode (' '));
            var a = document.createElementNS (HTML_NS, 'html:a');
            a.setAttribute ('href', '');
            a.setAttribute ('style', 'text-decoration: underline; color: blue');
            a.setAttribute ('onclick', 'signup("'+service.id+'"); return false');
            //a.appendChild (document.createEntityReference ('flock.favorites.firstRun.get'));
            a.appendChild (document.createTextNode ('Get an account'));
            p.appendChild (a);
          }
        }
        label.appendChild (p);
    }
    
    currentService = flock_getCharPref('flock.favorites.webservice.id');
    
    if (currentService != '') {
        $('service').selectedItem = $(currentService)
        currentUser = services[currentService].getUsername();
    }

    choose_service();
    origService = chosenService;
    if(origService) {
        origUsername = origService.getUsername();
        origPassword = origService.getPassword();
    }

    // initialize some check boxes from prefs
    $('sharefavorites').checked = 
        flock_getBoolPref ('flock.favorites.shareByDefault');
    $('privatedefault').selectedIndex = 
        flock_getBoolPref ('flock.favorites.privateByDefault')?1:0;
    $('copyfavorites').checked = 
        flock_getBoolPref ('flock.favorites.copyFavorites');
    $('copywarning').hidden = !$('copyfavorites').checked;
}

// the logic that determines if I'm switching to a new account
function amISwitching () {
    var svcid = $('service').selectedItem.id;
    var service = null;
    if (svcid != 'disabled') {
        service = services[svcid];
    }

    usernameChanged = false;

    if (!service) {
        // "disabled" has been selected
        return false;
    }
    
    // FIXME: how to handle switching from nothing to something

    if (currentService != service.id) {
        // switching between services
        return true;
    }

    if (service.getUsername () != $('username').value) {
        // the username has changed
        oldUsername = service.getUsername();
        oldPassword = service.getPassword();
        usernameChanged = true;
        return true;
    }

    return false;
}

// update the "copy favorites" ui based on the current state
function updateCopyState () {
    if (amISwitching()) {
        $('copyfavorites').checked = flock_getBoolPref ('flock.favorites.copyFavorites');
        $('copyfavorites').disabled = false;
    } else {
        $('copyfavorites').checked = false
        $('copyfavorites').disabled = true;
    }
}

// a service was chosen from the service list
function choose_service () {
    var svcid = $('service').selectedItem.id;
    var service = null;
    if (svcid != 'disabled') {
        service = services[svcid];
    }

    if (service) {
        $('username').disabled = false;
        $('username').value = service.getUsername ();
        $('password').disabled = false;
        $('password').value = service.getPassword ();
        $('sharefavorites').disabled = false;
        $('privatedefault').disabled = false;
    } else {
        $('username').value = '';
        $('username').disabled = true;
        $('password').value = ''
        $('password').disabled = true;
        $('sharefavorites').disabled = true;
        $('privatedefault').disabled = true;
    }
    
    updateCopyState ();

    chosenService = service;

    // FIXME: resize the window to fit everything
}

// sign up for an account, eh?
function signup (service) {
    window.open(brandingUtilities.buildWebsiteUrl('blog','favoritesSharing','getAccount',service));
}

function hasChanged() {
    if(origService != chosenService) return true;
    if(origPassword != $('password').value) return true;
    if(origUsername != $('username').value) return true;
    return false;
}
// what happens when you hit 'Cancel'
function dialogcancel () {
  if(hasChanged()) {
    if(origService) {
        origService.login (origUsername, origPassword, null); // The listener is at "null", so we will get a Javascript error "aListener has no properties". But we do it anyway because we want to close the window now.
    }
    chosenService = origService;
    commit();
    return true;
  }
  else
    return true;
}

// commit changes to the favorites sharing configuration
function commit () {
    // FIXME: check that we've logged in

    // update some prefs
    flock_setBoolPref ('flock.favorites.shareByDefault', $('sharefavorites').checked);
    flock_setBoolPref ('flock.favorites.privateByDefault', 
            $('privatedefault').selectedIndex==1);
    flock_setBoolPref ('flock.favorites.copyFavorites', $('copyfavorites').checked);

    if (!chosenService) {
        // sharing was turned off
        flock_setCharPref ('flock.favorites.webservice.id', '');
    } else if (chosenService != flock.favorites.service.webService) { 
        // the service was switched
        flock_setCharPref ('flock.favorites.webservice.id', chosenService.id);
    } else if (usernameChanged) {
        // if the username was changed
        // we need to toggle the service to trigger the re-sync 
        // we should rework the logic in flockFavoritesService so this isn't nececarry
        flock_setCharPref ('flock.favorites.webservice.id', '');
        flock_setCharPref ('flock.favorites.webservice.id', chosenService.id);
    }

    // set the first-run flag which now has a terrible name
    flock_setBoolPref ('flock.favorites.askedAboutSharing', true);

    return false;
}

function toggleServiceSelection(aTarget) {
        $('service').selectedItem = aTarget.parentNode.firstChild;
        choose_service();
}

//////////////////////////////////////////////////////////////////////////////
// wizard code
//////////////////////////////////////////////////////////////////////////////
var favoritesSharing = {};
favoritesSharing.chooseServicePage = {};

favoritesSharing.chooseServicePage.show = function() {
}

favoritesSharing.chooseServicePage.advanced = function() {
}

favoritesSharing.accountInfoPage = {};
favoritesSharing.accountInfoPage.show = function() {
    $("favoritesSharing").canAdvance = true;
    $("favoritesSharing").canRewind = true;
    if(!chosenService) {
        $("favoritesSharing").goTo("finishedPage");
    } else {
        var strBundle = document.getElementById('strBundle');
        var msg = strBundle.getString('flock.fav.share.infoTitle');
        var header = document.getAnonymousElementByAttribute($('favoritesSharing'), 'anonid','Header');
        header.setAttribute('label', msg.replace('%1',chosenService.name));
    }
}

favoritesSharing.accountInfoPage.advanced = function() {
}

favoritesSharing.loggingInPage = {};
favoritesSharing.loggingInPage.show = function() {
    $("favoritesSharing").canAdvance = false;
    $("favoritesSharing").canRewind = false;

    var listener = { };
    listener.onFailed = function (message) {
        if (cancelled) {
            return;
        }
        // we failed - tell the user why and take them back
        $("errorMessageContainer").removeAttribute('hidden');
        $("errorMsg").value = message;
        $("favoritesSharing").goTo("accountInfoPage");
    }
    listener.onSucceeded = function () {
        if (cancelled) {
            return;
        }
        $("favoritesSharing").goTo("optionsPage");
    }

    chosenService.login ($('username').value, $('password').value, listener);
}

favoritesSharing.loggingInPage.advanced = function() {
}

favoritesSharing.optionsPage = {};
favoritesSharing.optionsPage.show = function() {
    $("favoritesSharing").canAdvance = true;
    $("favoritesSharing").canRewind = true;
    if (firstrun) {
      $("switching").setAttribute('hidden','true');
    } else {
      $("switching").removeAttribute('hidden');
    }
}

favoritesSharing.optionsPage.advanced = function() {
}

favoritesSharing.finishedPage = {};
favoritesSharing.finishedPage.show = function() {
    $("favoritesSharing").canAdvance = true;
    if (chosenService) {
      $('setupService').removeAttribute('hidden');
      $('setupNoService').setAttribute('hidden','true');
    } else {
      $('setupNoService').removeAttribute('hidden')
      $('setupService').setAttribute('hidden','true');
    }
}

favoritesSharing.finishedPage.advanced = function() {
}

favoritesSharing.wizardFinish = function() {
    if(chosenService) {
        var strBundle = document.getElementById('strBundle');
        var msg = strBundle.getString('flock.fav.share.finishTitle');
        var header = document.getAnonymousElementByAttribute($('favoritesSharing'), 'anonid','Header');
        header.setAttribute('label', msg.replace('%1',chosenService.name));
    }
    commit ();
}
