// vim: foldmethod=marker

//
// BEGIN clav.mozdev.org MPL
// 
// Version: MPL 1.1/GPL 2.0/LGPL 2.1
// 
// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is Stephen Clavering code.
// 
// The Initial Developer of the Original Code is
// Stephen Clavering <mozilla@clav.me.uk>
// Portions created by the Initial Developer are Copyright (C) 2005
// the Initial Developer. All Rights Reserved.
// 
// Contributor(s):
// Flock Inc. <contact@flock.com> 
//
// Alternatively, the contents of this file may be used under the terms of
// either of the GNU General Public License Version 2 or later (the "GPL"),
// or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
// in which case the provisions of the GPL or the LGPL are applicable instead
// of those above. If you wish to allow use of your version of this file only
// under the terms of either the GPL or the LGPL, and not to allow others to
// use your version of this file under the terms of the MPL, indicate your
// decision by deleting the provisions above and replace them with the notice
// and other provisions required by the GPL or the LGPL. If you do not delete
// the provisions above, a recipient may use your version of this file under
// the terms of any one of the MPL, the GPL or the LGPL.
// 
// END clav.mozdev.org MPL
//


const kStopReloadNormalClass = "toolbarbutton-1 stopreload-reload";
const kStopReloadStopClass = "toolbarbutton-1 stopreload-stop";

// toolbox.customizeDone is now initialized.  Proceed with ultra sketchy
// customization notification override
function loadPhase2() {
    stopreloadinit_phase2();
    PhotoButtonListener.init();
}

function stopreloadinit_phase2() {
  var toolbox = document.getElementById("navigator-toolbox");
  toolbox.preStopreloadCustomizeDone = toolbox.customizeDone;
  toolbox.customizeDone = function(anythingChanged) {
    this.preStopreloadCustomizeDone(anythingChanged);
    if(anythingChanged) stopreload.init();
  };
}

var PhotoButtonListener = {
    PPSVC: Components.classes['@mozilla.org/rdf/datasource;1?name=flock-people-photo']
            .getService(Components.interfaces.flockIPhotoPeopleService),
    uninit: function() {
        this.PPSVC.removeListener(this);
    },
    init: function() {
        this.PPSVC.addListener(this);
        if(this.PPSVC.hasNew) {
            this.onNewPhotos();
        }
    },
    onUpdate: function() {
    },
    onNewPhotos: function() {
        $("photos-button").setAttribute("hasNew", "true");
    },
    onNewPhotosTouched: function() {
        $("photos-button").removeAttribute("hasNew");
    },
}

//first we must wait till load.  during this epoch, toolbox.customizeDone is
//initialized
top.addEventListener("load", function() {
  stopreload.init();
  window.setTimeout(loadPhase2, 0); //phase 2 see above
}, false);

top.addEventListener("unload", function() {
  PhotoButtonListener.uninit();
}, false);

var stopreload = {
  button: null,
  listenerAdded: false,

  init: function() {
    this.button = document.getElementById('stopreload-button');
    if(this.button) {
      if(this.listenerAdded) return;
      gBrowser.addProgressListener(this, Components.interfaces.nsIWebProgress.NOTIFY_STATE_ALL);
//      gBrowser.addProgressListener(this, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
      this.listenerAdded = true;
      // default label is 'Stop/Reload', for when it's on the palette
      this.button.setAttribute('label', this.button.getAttribute('reloadlabel'));
      this.button.setAttribute('tooltiptext', this.button.getAttribute('reloadtooltip'));
      this.button.stopOnCommand = false;
    } else {
      if(!this.listenerAdded) return;
      gBrowser.removeProgressListener(this);
      this.listenerAdded = false;
    }
  },

  oncommand: function(evt, button) {
    if(button.stopOnCommand) BrowserStop();
    else BrowserReloadSkipCache();
  },

  QueryInterface: function(aIID) {
    if(aIID.equals(Components.interfaces.nsIWebProgressListener)
        || aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
    const nsIWebProgressListener = Components.interfaces.nsIWebProgressListener;
    var btn = this.button;
    if(aStateFlags & nsIWebProgressListener.STATE_START) {
      btn.className = kStopReloadStopClass;
      btn.stopOnCommand = true;
      btn.setAttribute('label', btn.getAttribute('stoplabel'));
      btn.setAttribute('tooltiptext',btn.getAttribute('stoptooltip'));
    } else if(aStateFlags & nsIWebProgressListener.STATE_STOP) {
      btn.className = kStopReloadNormalClass;
      btn.stopOnCommand = false;
      btn.setAttribute('label', btn.getAttribute('reloadlabel'));
      btn.setAttribute('tooltiptext', btn.getAttribute('reloadtooltip'));
    }
  },

  onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {},
  onLocationChange: function(aWebProgress, aRequest, aLocation) {},
  onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {},
  onSecurityChange: function(aWebProgress, aRequest, aState) {},

  // tabbrowser.xml calls this for all registered progress listeners,
  // even though it is not part of the nsIWebProgressListener interface
  onLinkIconAvailable: function(href) {}
}

var blogButtonDNDObserver =
{
    onDragOver: function(aEvent, aFlavor, aSession) {
    },

    onDrop: function(aEvent, aDropData, aSession) {
        flock_goDoCommand("cmd_flock_blogBlogThis");
    },

    onDragStart: function(aEvent, aTransferData, aAction) {
    },

    getSupportedFlavours: function() {
        var flavors = new FlavourSet();
        flavors.appendFlavour("text/html");
        return flavors;
    }
}
