/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is SpellBound code.
 *
 * The Initial Developer of the Original Code is
 * Robert Strong <moz@exchangecode.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var sbOverlay = {
  isLoaded: false,                  // has the load function setup the overlay
  isSpellCheckerInstalled: false,   // is the spellchecker.xpt registered
  lastFocusedNode: null,            // last node that had focus

  load: function() {
    if (sbOverlay.isLoaded)
      return;

    if ("@mozilla.org/spellchecker;1" in Components.classes) {
      sbOverlay.isSpellCheckerInstalled = true;
      document.getElementById("contentAreaContextMenu")
        .addEventListener("popupshowing", sbOverlay.setContext, false);
      document.addEventListener("click", sbOverlay.setEnabled, false);
      document.addEventListener("keyup", sbOverlay.setEnabled, false);
      getBrowser().addEventListener("click", sbOverlay.onClickHandler, true); 
      sbOverlay.componentInit();
      if (getSBPrefs().getBoolPref("toolsmenuitem.hidden", true) == false)
        document.getElementById("spellbound-toolsmenu").hidden = false;
    } else {
      document.getElementById("menu_sbcheckspelling").hidden = true;
      document.getElementById("sep_sbcheckspelling").hidden = true;
    }
    sbOverlay.isLoaded = true;
  },

  setContext: function(evt) {
    var nType = sbOverlay.isNodeSpellCheckable(gContextMenu.target);
    document.getElementById("menu_sbcheckspelling").hidden = !nType;
    document.getElementById("sep_sbcheckspelling").hidden = !nType;
  },

  onClickHandler: function(evt) {
    var node = evt.originalTarget;
    if (!node.getAttribute
    || !node.hasAttribute("id")
    || node.getAttribute("id").indexOf("spellboundSC") != 0)
      return null;

    evt.stopPropagation();
    evt.preventDefault();
    sbOverlay.openSpellCheck();
    // place focus on the original elemet
    sbOverlay.lastFocusedNode.focus();
    return null;
  },

  // enables cmd_spellbound based on whether a node can be spell checked
  // TODO: add enalbing when switching tabs for midas
  setEnabled: function(evt) {
    var node = evt.target;
    if (!node || !node.nodeName) {
      document.getElementById("cmd_spellbound").setAttribute("disabled", "true");
      return;
    }

    if (node.nodeName == "tab" || node.nodeName == "tabbrowser") {
      // when a tab in an extension is clicked (e.g. QuickNote, etc.)
      setTimeout(sbOverlay.setEnabledForTab, 0)
    } else {
      if (sbOverlay.isNodeSpellCheckable(document.commandDispatcher.focusedElement))
        node = document.commandDispatcher.focusedElement;
      if (!node || !node.nodeName) {
        document.getElementById("cmd_spellbound").setAttribute("disabled", "true");
        return;
      }
      // don't update lastFocusedNode when the spellbound  button is clicked
      if (node.hasAttribute("id")
      && node.getAttribute("id") != "spellbound-button"
      || !node.hasAttribute("id")) {
        var nType = sbOverlay.isNodeSpellCheckable(node);
        if (nType > 0) {
          document.getElementById("cmd_spellbound").removeAttribute("disabled");
          sbOverlay.lastFocusedNode = node;
        } else {
          document.getElementById("cmd_spellbound").setAttribute("disabled", "true");
        }
      }
    }
  },

  // TODO: add enalbing when switching tabs for midas
  setEnabledForTab: function() {
    var node = document.commandDispatcher.focusedElement;
    if (!node || !node.nodeName) {
      document.getElementById("cmd_spellbound").setAttribute("disabled", "true");
      return;
    }

    // don't update lastFocusedNode when the spellbound  button is clicked
    if (node.hasAttribute("id")
    && node.getAttribute("id") != "spellbound-button"
    || !node.hasAttribute("id")) {
      var nType = sbOverlay.isNodeSpellCheckable(node);
      if (nType > 0) {
        document.getElementById("cmd_spellbound").removeAttribute("disabled");
        // don't update lastFocusedNode for anything other than form elements
        if (nType < 200)
          sbOverlay.lastFocusedNode = node;
      } else {
        document.getElementById("cmd_spellbound").setAttribute("disabled", "true");
      }
    }
  },

  openSpellCheck: function() {
    var params;
    var nType = sbOverlay.isNodeSpellCheckable(sbOverlay.lastFocusedNode);

    if (sbOverlay.isSpellCheckerInstalled && nType > 0) {
      var args = [];
      args[0] = sbOverlay.lastFocusedNode;

      switch(nType) {
        case 200: // editor
          params = "chrome,close,titlebar";
          break;
        default:  // all others
          params = "chrome,close,titlebar,resizable";
          break;
      }
      if (getSBPrefs().getBoolPref("spellcheck.modal", true))
        params += ",modal"

      window.openDialog("chrome://spellbound/content/SBSpellCheck.xul",
          "_blank", params, false, true, true, args);
    }
  },

  isNodeSpellCheckable: function(node) {
    if (!node || typeof node.nodeName == "undefined"
    || node.hasAttribute("readonly"))
      return 0;

    var nType = sbGetNodeType(node);

    if (nType == 0)
      sbOverlay.lastFocusedNode = null;

    return nType;
  },

  componentInit: function() {
    if (typeof(Components.classes["@mozilla.org/spellbound;1"]) == "undefined") {
      // Component registration
      var compMgr = Components.manager
        .QueryInterface(Components.interfaces.nsIComponentRegistrar);
      try {
        compMgr.registerFactory(Components.ID("{F008FCB9-9CA4-4817-953A-09F4C234B48E}"),
          "SpellBound-SpellChecker", "@mozilla.org/spellbound;1", spellboundFactory);
      } catch (e) { dump(e); }
    }
  }

};

// This component's soul purpose is to provide an easy
// way for extensions know if SpellBound is installed
var SpellBoundService = {
  QueryInterface: function(iid) {
    if (!iid.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

var spellboundFactory = {
  // nsIFactory interface implementation
   createInstance: function(outer, iid) {
    if (outer) throw Components.results.NS_ERROR_NO_AGGREGATION;
    if (!iid.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_INVALID_ARG;
    return SpellBoundService;
  }
}

// called twice to try to work around Bug 174320. If it has already
// been called sbOverlay.isLoaded is true so it won't call it twice
window.addEventListener("load",sbOverlay.load,false);
if (!sbOverlay.isLoaded)
  window.addEventListener("load",sbOverlay.load,false);
